unit ProducerConsumerThreads;

interface

uses
  Classes;

type
  TProducerThread = class(TThread)
  private
    FStartNo: integer;
  protected
    procedure Execute; override;
  public
    property StartNo : integer read FStartNo write FStartNo;
  end;

  TConsumerThread = class(TThread)
    procedure Execute; override;
  end;

implementation

uses
  Queues,Libc;

const
  MAX_QUEUE_SIZE = 10;

type
  TMyQueuedItem = class
  public
    number : integer;
  end;

var
  TheQueue : TThreadSafeQueue;

{ TConsumerThread }

procedure TConsumerThread.Execute;
var
  obj : TMyQueuedItem;

begin
  while not Terminated do
  begin
    writeln(pthread_self,' Removing from the queue..');
    obj := TMyQueuedItem(TheQueue.GetObject);
    writeln(pthread_self,' after removing the item.. number is ',obj.number);

    //pthread_yield is part of glibc v 2.2
    //pthread_yield;

    sleep(2);
  end;
end;

{ TProducerThread }

procedure TProducerThread.Execute;
var
  obj : TMyQueuedItem;

begin
  while not Terminated do
  begin
    // create an item for the queue
    writeln(pthread_self,' Adding number ',FStartNo,' to the queue');
    obj := TMyQueuedItem.Create;
    obj.number := FStartNo;
    inc(FStartNo);

    // add this to the queue
    writeln(pthread_self,' Doing the add..');
    TheQueue.AddObject(obj);
    writeln(pthread_self,' Added to queue..');

    pthread_yield;
    sleep(2);
  end;
end;


initialization
  TheQueue := TThreadSafeQueue.Create(qtLifo,MAX_QUEUE_SIZE);
  TheQueue.Debug := true;
finalization
  TheQueue.Free;
end.
